@@ -14,22 +14,19 @@
  * limitations under the License.
  */
 
+#include "bson/bson.h"
+#include "mongoc/mongoc.h"
+
 #include <php.h>
 #include <Zend/zend_interfaces.h>
 
-#ifdef HAVE_CONFIG_H
-#include "config.h"
-#endif
-
 #include "php_array_api.h"
-#include "phongo_compat.h"
-#include "php_phongo.h"
-#include "php_bson.h"
 
-#if PHP_VERSION_ID < 70200
-#include <ext/spl/spl_iterators.h>
-#define zend_ce_countable spl_ce_Countable
-#endif /* PHP_VERSION_ID < 70200 */
+#include "php_phongo.h"
+#include "phongo_bson_encode.h"
+#include "phongo_error.h"
+
+#include "MongoDB/WriteConcern.h"
 
 #define PHONGO_BULKWRITE_BYPASS_UNSET -1
 
@@ -531,10 +528,14 @@ static PHP_METHOD(BulkWrite, count)
 } /* }}} */
 
 /* {{{ MongoDB\Driver\BulkWrite function entries */
+/* clang-format off */
 ZEND_BEGIN_ARG_INFO_EX(ai_BulkWrite___construct, 0, 0, 0)
 	ZEND_ARG_ARRAY_INFO(0, options, 1)
 ZEND_END_ARG_INFO()
 
+ZEND_BEGIN_ARG_WITH_TENTATIVE_RETURN_TYPE_INFO_EX(ai_BulkWrite_count, 0, 0, IS_LONG, 0)
+ZEND_END_ARG_INFO()
+
 ZEND_BEGIN_ARG_INFO_EX(ai_BulkWrite_insert, 0, 0, 1)
 	ZEND_ARG_INFO(0, document)
 ZEND_END_ARG_INFO()
@@ -554,16 +555,15 @@ ZEND_BEGIN_ARG_INFO_EX(ai_BulkWrite_void, 0, 0, 0)
 ZEND_END_ARG_INFO()
 
 static zend_function_entry php_phongo_bulkwrite_me[] = {
-	/* clang-format off */
 	PHP_ME(BulkWrite, __construct, ai_BulkWrite___construct, ZEND_ACC_PUBLIC | ZEND_ACC_FINAL)
 	PHP_ME(BulkWrite, insert, ai_BulkWrite_insert, ZEND_ACC_PUBLIC | ZEND_ACC_FINAL)
 	PHP_ME(BulkWrite, update, ai_BulkWrite_update, ZEND_ACC_PUBLIC | ZEND_ACC_FINAL)
 	PHP_ME(BulkWrite, delete, ai_BulkWrite_delete, ZEND_ACC_PUBLIC | ZEND_ACC_FINAL)
-	PHP_ME(BulkWrite, count, ai_BulkWrite_void, ZEND_ACC_PUBLIC | ZEND_ACC_FINAL)
+	PHP_ME(BulkWrite, count, ai_BulkWrite_count, ZEND_ACC_PUBLIC | ZEND_ACC_FINAL)
 	ZEND_NAMED_ME(__wakeup, PHP_FN(MongoDB_disabled___wakeup), ai_BulkWrite_void, ZEND_ACC_PUBLIC | ZEND_ACC_FINAL)
 	PHP_FE_END
-	/* clang-format on */
 };
+/* clang-format on */
 /* }}} */
 
 /* {{{ MongoDB\Driver\BulkWrite object handlers */
@@ -594,9 +594,7 @@ static void php_phongo_bulkwrite_free_object(zend_object* object) /* {{{ */
 
 static zend_object* php_phongo_bulkwrite_create_object(zend_class_entry* class_type) /* {{{ */
 {
-	php_phongo_bulkwrite_t* intern = NULL;
-
-	intern = PHONGO_ALLOC_OBJECT_T(php_phongo_bulkwrite_t, class_type);
+	php_phongo_bulkwrite_t* intern = zend_object_alloc(sizeof(php_phongo_bulkwrite_t), class_type);
 
 	zend_object_std_init(&intern->std, class_type);
 	object_properties_init(&intern->std, class_type);
@@ -675,12 +673,3 @@ void php_phongo_bulkwrite_init_ce(INIT_FUNC_ARGS) /* {{{ */
 
 	zend_class_implements(php_phongo_bulkwrite_ce, 1, zend_ce_countable);
 } /* }}} */
-
-/*
- * Local variables:
- * tab-width: 4
- * c-basic-offset: 4
- * End:
- * vim600: noet sw=4 ts=4 fdm=marker
- * vim<600: noet sw=4 ts=4
- */
