@@ -15,16 +15,16 @@
  */
 
 #include <php.h>
-#include <Zend/zend_interfaces.h>
 #include <ext/spl/spl_iterators.h>
+#include <Zend/zend_interfaces.h>
 
-#ifdef HAVE_CONFIG_H
-#include "config.h"
-#endif
-
-#include "phongo_compat.h"
 #include "php_phongo.h"
-#include "php_bson.h"
+#include "phongo_bson.h"
+#include "phongo_client.h"
+#include "phongo_error.h"
+
+#include "MongoDB/Cursor.h"
+#include "MongoDB/Server.h"
 
 zend_class_entry* php_phongo_cursor_ce;
 
@@ -351,6 +351,22 @@ PHP_METHOD(Cursor, rewind)
 }
 
 /* {{{ MongoDB\Driver\Cursor function entries */
+/* clang-format off */
+ZEND_BEGIN_ARG_WITH_TENTATIVE_RETURN_TYPE_INFO_EX(ai_Cursor_current, 0, 0, IS_MIXED, 0)
+ZEND_END_ARG_INFO()
+
+ZEND_BEGIN_ARG_WITH_TENTATIVE_RETURN_TYPE_INFO_EX(ai_Cursor_key, 0, 0, IS_MIXED, 0)
+ZEND_END_ARG_INFO()
+
+ZEND_BEGIN_ARG_WITH_TENTATIVE_RETURN_TYPE_INFO_EX(ai_Cursor_next, 0, 0, IS_VOID, 0)
+ZEND_END_ARG_INFO()
+
+ZEND_BEGIN_ARG_WITH_TENTATIVE_RETURN_TYPE_INFO_EX(ai_Cursor_valid, 0, 0, _IS_BOOL, 0)
+ZEND_END_ARG_INFO()
+
+ZEND_BEGIN_ARG_WITH_TENTATIVE_RETURN_TYPE_INFO_EX(ai_Cursor_rewind, 0, 0, IS_VOID, 0)
+ZEND_END_ARG_INFO()
+
 ZEND_BEGIN_ARG_INFO_EX(ai_Cursor_setTypeMap, 0, 0, 1)
 	ZEND_ARG_ARRAY_INFO(0, typemap, 0)
 ZEND_END_ARG_INFO()
@@ -359,24 +375,23 @@ ZEND_BEGIN_ARG_INFO_EX(ai_Cursor_void, 0, 0, 0)
 ZEND_END_ARG_INFO()
 
 static zend_function_entry php_phongo_cursor_me[] = {
-	/* clang-format off */
 	PHP_ME(Cursor, setTypeMap, ai_Cursor_setTypeMap, ZEND_ACC_PUBLIC | ZEND_ACC_FINAL)
 	PHP_ME(Cursor, toArray, ai_Cursor_void, ZEND_ACC_PUBLIC | ZEND_ACC_FINAL)
 	PHP_ME(Cursor, getId, ai_Cursor_void, ZEND_ACC_PUBLIC | ZEND_ACC_FINAL)
 	PHP_ME(Cursor, getServer, ai_Cursor_void, ZEND_ACC_PUBLIC | ZEND_ACC_FINAL)
 	PHP_ME(Cursor, isDead, ai_Cursor_void, ZEND_ACC_PUBLIC | ZEND_ACC_FINAL)
 
-	PHP_ME(Cursor, current, ai_Cursor_void, ZEND_ACC_PUBLIC)
-	PHP_ME(Cursor, key, ai_Cursor_void, ZEND_ACC_PUBLIC)
-	PHP_ME(Cursor, next, ai_Cursor_void, ZEND_ACC_PUBLIC)
-	PHP_ME(Cursor, valid, ai_Cursor_void, ZEND_ACC_PUBLIC)
-	PHP_ME(Cursor, rewind, ai_Cursor_void, ZEND_ACC_PUBLIC)
+	PHP_ME(Cursor, current, ai_Cursor_current, ZEND_ACC_PUBLIC)
+	PHP_ME(Cursor, key, ai_Cursor_key, ZEND_ACC_PUBLIC)
+	PHP_ME(Cursor, next, ai_Cursor_next, ZEND_ACC_PUBLIC)
+	PHP_ME(Cursor, valid, ai_Cursor_valid, ZEND_ACC_PUBLIC)
+	PHP_ME(Cursor, rewind, ai_Cursor_rewind, ZEND_ACC_PUBLIC)
 
 	ZEND_NAMED_ME(__construct, PHP_FN(MongoDB_disabled___construct), ai_Cursor_void, ZEND_ACC_PRIVATE | ZEND_ACC_FINAL)
 	ZEND_NAMED_ME(__wakeup, PHP_FN(MongoDB_disabled___wakeup), ai_Cursor_void, ZEND_ACC_PUBLIC | ZEND_ACC_FINAL)
 	PHP_FE_END
-	/* clang-format on */
 };
+/* clang-format on */
 /* }}} */
 
 /* {{{ MongoDB\Driver\Cursor object handlers */
@@ -432,9 +447,7 @@ static void php_phongo_cursor_free_object(zend_object* object) /* {{{ */
 
 static zend_object* php_phongo_cursor_create_object(zend_class_entry* class_type) /* {{{ */
 {
-	php_phongo_cursor_t* intern = NULL;
-
-	intern = PHONGO_ALLOC_OBJECT_T(php_phongo_cursor_t, class_type);
+	php_phongo_cursor_t* intern = zend_object_alloc(sizeof(php_phongo_cursor_t), class_type);
 
 	zend_object_std_init(&intern->std, class_type);
 	object_properties_init(&intern->std, class_type);
@@ -538,11 +551,26 @@ void php_phongo_cursor_init_ce(INIT_FUNC_ARGS) /* {{{ */
 	php_phongo_handler_cursor.offset         = XtOffsetOf(php_phongo_cursor_t, std);
 } /* }}} */
 
-/*
- * Local variables:
- * tab-width: 4
- * c-basic-offset: 4
- * End:
- * vim600: noet sw=4 ts=4 fdm=marker
- * vim<600: noet sw=4 ts=4
- */
+/* Advance the cursor and return whether there is an error. On error, false is
+ * returned and an exception is thrown. */
+bool phongo_cursor_advance_and_check_for_error(mongoc_cursor_t* cursor) /* {{{ */
+{
+	const bson_t* doc = NULL;
+
+	if (!mongoc_cursor_next(cursor, &doc)) {
+		bson_error_t error = { 0 };
+
+		/* Check for connection related exceptions */
+		if (EG(exception)) {
+			return false;
+		}
+
+		/* Could simply be no docs, which is not an error */
+		if (mongoc_cursor_error_document(cursor, &error, &doc)) {
+			phongo_throw_exception_from_bson_error_t_and_reply(&error, doc);
+			return false;
+		}
+	}
+
+	return true;
+} /* }}} */
