@@ -1,5 +1,5 @@
 /*
- * Copyright 2015-2017 MongoDB, Inc.
+ * Copyright 2015-present MongoDB, Inc.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -17,14 +17,11 @@
 #include <php.h>
 #include <ext/spl/spl_exceptions.h>
 
-#ifdef HAVE_CONFIG_H
-#include "config.h"
-#endif
-
-#include "phongo_compat.h"
-#include "php_phongo.h"
 #include "php_array_api.h"
 
+#include "php_phongo.h"
+#include "phongo_error.h"
+
 zend_class_entry* php_phongo_runtimeexception_ce;
 
 static bool php_phongo_has_string_array_element(zval* labels, char* label)
@@ -56,16 +53,20 @@ static bool php_phongo_has_string_array_element(zval* labels, char* label)
    Returns whether a specific error label has been set */
 static PHP_METHOD(RuntimeException, hasErrorLabel)
 {
-	char*  label;
-	size_t label_len;
-	zval*  error_labels;
-	zval   rv;
+	zend_error_handling error_handling;
+	char*               label;
+	size_t              label_len;
+	zval*               error_labels;
+	zval                rv;
 
+	zend_replace_error_handling(EH_THROW, phongo_exception_from_phongo_domain(PHONGO_ERROR_INVALID_ARGUMENT), &error_handling);
 	if (zend_parse_parameters(ZEND_NUM_ARGS(), "s", &label, &label_len) == FAILURE) {
+		zend_restore_error_handling(&error_handling);
 		return;
 	}
+	zend_restore_error_handling(&error_handling);
 
-	error_labels = zend_read_property(php_phongo_runtimeexception_ce, getThis(), ZEND_STRL("errorLabels"), 0, &rv);
+	error_labels = zend_read_property(php_phongo_runtimeexception_ce, PHONGO_COMPAT_OBJ_P(getThis()), ZEND_STRL("errorLabels"), 0, &rv);
 
 	RETURN_BOOL(php_phongo_has_string_array_element(error_labels, label));
 } /* }}} */
@@ -93,12 +94,3 @@ void php_phongo_runtimeexception_init_ce(INIT_FUNC_ARGS) /* {{{ */
 
 	zend_declare_property_null(php_phongo_runtimeexception_ce, ZEND_STRL("errorLabels"), ZEND_ACC_PROTECTED);
 } /* }}} */
-
-/*
- * Local variables:
- * tab-width: 4
- * c-basic-offset: 4
- * End:
- * vim600: noet sw=4 ts=4 fdm=marker
- * vim<600: noet sw=4 ts=4
- */
