@@ -14,26 +14,31 @@
  * limitations under the License.
  */
 
-#include <php.h>
-#include <Zend/zend_hash.h>
-#include <Zend/zend_interfaces.h>
-#include <ext/standard/file.h>
+#include "bson/bson.h"
+#include "mongoc/mongoc.h"
 
-#ifdef HAVE_CONFIG_H
-#include "config.h"
-#endif
+#include <php.h>
+#include <ext/standard/file.h>
+#include <Zend/zend_interfaces.h>
 
 #include "php_array_api.h"
-#include "phongo_compat.h"
+
 #include "php_phongo.h"
-#include "Session.h"
-#include "src/phongo_apm.h"
+#include "phongo_apm.h"
+#include "phongo_client.h"
+#include "phongo_error.h"
+#include "phongo_execute.h"
+#include "phongo_util.h"
+
+#include "MongoDB/ClientEncryption.h"
+#include "MongoDB/ReadConcern.h"
+#include "MongoDB/ReadPreference.h"
+#include "MongoDB/Server.h"
+#include "MongoDB/Session.h"
+#include "MongoDB/WriteConcern.h"
 
 #define PHONGO_MANAGER_URI_DEFAULT "mongodb://127.0.0.1/"
 
-#undef MONGOC_LOG_DOMAIN
-#define MONGOC_LOG_DOMAIN "PHONGO"
-
 /**
  * Manager abstracts a cluster of Server objects (i.e. socket connections).
  *
@@ -83,9 +88,7 @@ static bool php_phongo_manager_merge_context_options(zval* zdriverOptions) /* {{
 	 * scope, leading to a COW violation in zend_hash_merge and
 	 * zend_symtable_str_del (called by php_array_unsetc). This macro allows
 	 * that violation in debug mode and is a NOOP when in non-debug. */
-#if PHP_VERSION_ID >= 70200
 	HT_ALLOW_COW_VIOLATION(Z_ARRVAL_P(zdriverOptions));
-#endif
 
 	php_error_docref(NULL, E_DEPRECATED, "The \"context\" driver option is deprecated.");
 
@@ -308,10 +311,8 @@ static PHP_METHOD(Manager, addSubscriber)
    Return a ClientEncryption instance */
 static PHP_METHOD(Manager, createClientEncryption)
 {
-	zend_error_handling            error_handling;
-	php_phongo_manager_t*          intern;
-	php_phongo_clientencryption_t* clientencryption;
-	zval*                          options;
+	zend_error_handling error_handling;
+	zval*               options;
 
 	zend_replace_error_handling(EH_THROW, phongo_exception_from_phongo_domain(PHONGO_ERROR_INVALID_ARGUMENT), &error_handling);
 	if (zend_parse_parameters(ZEND_NUM_ARGS(), "a", &options) == FAILURE) {
@@ -320,13 +321,8 @@ static PHP_METHOD(Manager, createClientEncryption)
 	}
 	zend_restore_error_handling(&error_handling);
 
-	intern = Z_MANAGER_OBJ_P(getThis());
-
-	object_init_ex(return_value, php_phongo_clientencryption_ce);
-	clientencryption = Z_CLIENTENCRYPTION_OBJ_P(return_value);
-
 	/* An exception will be thrown on error. */
-	phongo_clientencryption_init(clientencryption, getThis(), options);
+	phongo_clientencryption_init(return_value, getThis(), options);
 } /* }}} */
 
 /* {{{ proto MongoDB\Driver\Cursor MongoDB\Driver\Manager::executeCommand(string $db, MongoDB\Driver\Command $command[, array $options = null])
@@ -721,25 +717,22 @@ static PHP_METHOD(Manager, removeSubscriber)
 	phongo_apm_remove_subscriber(intern->subscribers, subscriber);
 } /* }}} */
 
-/* {{{ proto MongoDB\Driver\Server MongoDB\Driver\Manager::selectServers(MongoDB\Driver\ReadPreference $readPreference)
-   Returns a suitable Server for the given ReadPreference */
+/* {{{ proto MongoDB\Driver\Server MongoDB\Driver\Manager::selectServers([MongoDB\Driver\ReadPreference $readPreference = null])
+   Selects a Server for the given ReadPreference (default: primary). */
 static PHP_METHOD(Manager, selectServer)
 {
-	zend_error_handling   error_handling;
 	php_phongo_manager_t* intern;
 	zval*                 zreadPreference = NULL;
 	uint32_t              server_id       = 0;
 
+	PHONGO_PARSE_PARAMETERS_START(0, 1)
+	Z_PARAM_OPTIONAL
+	Z_PARAM_OBJECT_OF_CLASS_OR_NULL(zreadPreference, php_phongo_readpreference_ce)
+	PHONGO_PARSE_PARAMETERS_END();
+
 	intern = Z_MANAGER_OBJ_P(getThis());
 
-	zend_replace_error_handling(EH_THROW, phongo_exception_from_phongo_domain(PHONGO_ERROR_INVALID_ARGUMENT), &error_handling);
-	if (zend_parse_parameters(ZEND_NUM_ARGS(), "O", &zreadPreference, php_phongo_readpreference_ce) == FAILURE) {
-		zend_restore_error_handling(&error_handling);
-		return;
-	}
-	zend_restore_error_handling(&error_handling);
-
-	if (!php_phongo_manager_select_server(false, true, zreadPreference, NULL, intern->client, &server_id)) {
+	if (!php_phongo_manager_select_server(false, false, zreadPreference, NULL, intern->client, &server_id)) {
 		/* Exception should already have been thrown */
 		return;
 	}
@@ -805,6 +798,18 @@ static PHP_METHOD(Manager, startSession)
 		}
 	}
 
+	if (options && php_array_existsc(options, "snapshot")) {
+		if (!cs_opts) {
+			cs_opts = mongoc_session_opts_new();
+		}
+		mongoc_session_opts_set_snapshot(cs_opts, php_array_fetchc_bool(options, "snapshot"));
+	}
+
+	if (cs_opts && mongoc_session_opts_get_causal_consistency(cs_opts) && mongoc_session_opts_get_snapshot(cs_opts)) {
+		phongo_throw_exception(PHONGO_ERROR_INVALID_ARGUMENT, "Only one of \"causalConsistency\" and \"snapshot\" can be enabled");
+		goto cleanup;
+	}
+
 	/* If the Manager was created in a different process, reset the client so
 	 * that its session pool is cleared. This will ensure that we do not re-use
 	 * a server session (i.e. LSID) created by a parent process. */
@@ -867,8 +872,8 @@ ZEND_BEGIN_ARG_INFO_EX(ai_Manager_removeSubscriber, 0, 0, 1)
 	ZEND_ARG_OBJ_INFO(0, subscriber, MongoDB\\Driver\\Monitoring\\Subscriber, 0)
 ZEND_END_ARG_INFO()
 
-ZEND_BEGIN_ARG_INFO_EX(ai_Manager_selectServer, 0, 0, 1)
-	ZEND_ARG_OBJ_INFO(0, readPreference, MongoDB\\Driver\\ReadPreference, 0)
+ZEND_BEGIN_ARG_INFO_EX(ai_Manager_selectServer, 0, 0, 0)
+	ZEND_ARG_OBJ_INFO(0, readPreference, MongoDB\\Driver\\ReadPreference, 1)
 ZEND_END_ARG_INFO()
 
 ZEND_BEGIN_ARG_INFO_EX(ai_Manager_startSession, 0, 0, 0)
@@ -911,11 +916,6 @@ static void php_phongo_manager_free_object(zend_object* object) /* {{{ */
 
 	zend_object_std_dtor(&intern->std);
 
-	/* Update the request-scoped Manager registry. The return value is ignored
-	 * because it's possible that the Manager was never registered due to a
-	 * constructor exception. */
-	php_phongo_manager_unregister(intern);
-
 	if (intern->client) {
 		/* Request-scoped clients will be removed from the registry and
 		 * destroyed. This is a NOP for persistent clients. The return value is
@@ -925,6 +925,15 @@ static void php_phongo_manager_free_object(zend_object* object) /* {{{ */
 		php_phongo_client_unregister(intern);
 	}
 
+	/* Update the request-scoped Manager registry. The return value is ignored
+	 * because it's possible that the Manager was never registered due to a
+	 * constructor exception.
+	 *
+	 * Note: this is done after unregistering a request-scoped client to ensure
+	 * APM events can be observed by per-client subscribers, which are collected
+	 * in phongo_apm_get_subscribers_to_notify. */
+	php_phongo_manager_unregister(intern);
+
 	if (intern->client_hash) {
 		efree(intern->client_hash);
 	}
@@ -943,9 +952,7 @@ static void php_phongo_manager_free_object(zend_object* object) /* {{{ */
 
 static zend_object* php_phongo_manager_create_object(zend_class_entry* class_type) /* {{{ */
 {
-	php_phongo_manager_t* intern = NULL;
-
-	intern = PHONGO_ALLOC_OBJECT_T(php_phongo_manager_t, class_type);
+	php_phongo_manager_t* intern = zend_object_alloc(sizeof(php_phongo_manager_t), class_type);
 
 	zend_object_std_init(&intern->std, class_type);
 	object_properties_init(&intern->std, class_type);
@@ -979,7 +986,7 @@ static HashTable* php_phongo_manager_get_debug_info(phongo_compat_object_handler
 	for (i = 0; i < n; i++) {
 		zval obj;
 
-		if (!php_phongo_server_to_zval(&obj, sds[i])) {
+		if (!php_phongo_server_to_zval(&obj, intern->client, sds[i])) {
 			/* Exception already thrown */
 			zval_ptr_dtor(&obj);
 			zval_ptr_dtor(&cluster);
@@ -1013,12 +1020,3 @@ void php_phongo_manager_init_ce(INIT_FUNC_ARGS) /* {{{ */
 	php_phongo_handler_manager.free_obj       = php_phongo_manager_free_object;
 	php_phongo_handler_manager.offset         = XtOffsetOf(php_phongo_manager_t, std);
 } /* }}} */
-
-/*
- * Local variables:
- * tab-width: 4
- * c-basic-offset: 4
- * End:
- * vim600: noet sw=4 ts=4 fdm=marker
- * vim<600: noet sw=4 ts=4
- */
