@@ -14,15 +14,17 @@
  * limitations under the License.
  */
 
+#include "bson/bson.h"
+
 #include <php.h>
 #include <Zend/zend_interfaces.h>
 
-#ifdef HAVE_CONFIG_H
-#include "config.h"
-#endif
-
-#include "phongo_compat.h"
 #include "php_phongo.h"
+#include "phongo_bson_encode.h"
+#include "phongo_error.h"
+
+#include "BSON/ObjectId.h"
+#include "MongoDB/Server.h"
 
 zend_class_entry* php_phongo_commandstartedevent_ce;
 
@@ -152,6 +154,21 @@ PHP_METHOD(CommandStartedEvent, getServer)
 	phongo_server_init(return_value, &intern->manager, intern->server_id);
 } /* }}} */
 
+/* {{{ proto MongoDB\BSON\ObjectId|null CommandStartedEvent::getServiceId()
+   Returns the event's service ID */
+PHP_METHOD(CommandStartedEvent, getServiceId)
+{
+	php_phongo_commandstartedevent_t* intern = Z_COMMANDSTARTEDEVENT_OBJ_P(getThis());
+
+	PHONGO_PARSE_PARAMETERS_NONE();
+
+	if (!intern->has_service_id) {
+		RETURN_NULL();
+	}
+
+	phongo_objectid_init(return_value, &intern->service_id);
+} /* }}} */
+
 /**
  * Event thrown when a command has started to execute.
  *
@@ -171,6 +188,7 @@ static zend_function_entry php_phongo_commandstartedevent_me[] = {
 	PHP_ME(CommandStartedEvent, getOperationId, ai_CommandStartedEvent_void, ZEND_ACC_PUBLIC | ZEND_ACC_FINAL)
 	PHP_ME(CommandStartedEvent, getRequestId, ai_CommandStartedEvent_void, ZEND_ACC_PUBLIC | ZEND_ACC_FINAL)
 	PHP_ME(CommandStartedEvent, getServer, ai_CommandStartedEvent_void, ZEND_ACC_PUBLIC | ZEND_ACC_FINAL)
+	PHP_ME(CommandStartedEvent, getServiceId, ai_CommandStartedEvent_void, ZEND_ACC_PUBLIC | ZEND_ACC_FINAL)
 	ZEND_NAMED_ME(__wakeup, PHP_FN(MongoDB_disabled___wakeup), ai_CommandStartedEvent_void, ZEND_ACC_PUBLIC | ZEND_ACC_FINAL)
 	PHP_FE_END
 	/* clang-format on */
@@ -205,9 +223,7 @@ static void php_phongo_commandstartedevent_free_object(zend_object* object) /* {
 
 static zend_object* php_phongo_commandstartedevent_create_object(zend_class_entry* class_type) /* {{{ */
 {
-	php_phongo_commandstartedevent_t* intern = NULL;
-
-	intern = PHONGO_ALLOC_OBJECT_T(php_phongo_commandstartedevent_t, class_type);
+	php_phongo_commandstartedevent_t* intern = zend_object_alloc(sizeof(php_phongo_commandstartedevent_t), class_type);
 
 	zend_object_std_init(&intern->std, class_type);
 	object_properties_init(&intern->std, class_type);
@@ -253,6 +269,15 @@ static HashTable* php_phongo_commandstartedevent_get_debug_info(phongo_compat_ob
 		ADD_ASSOC_ZVAL_EX(&retval, "server", &server);
 	}
 
+	if (intern->has_service_id) {
+		zval service_id;
+
+		phongo_objectid_init(&service_id, &intern->service_id);
+		ADD_ASSOC_ZVAL_EX(&retval, "serviceId", &service_id);
+	} else {
+		ADD_ASSOC_NULL_EX(&retval, "serviceId");
+	}
+
 done:
 	return Z_ARRVAL(retval);
 } /* }}} */
@@ -277,12 +302,3 @@ void php_phongo_commandstartedevent_init_ce(INIT_FUNC_ARGS) /* {{{ */
 
 	return;
 } /* }}} */
-
-/*
- * Local variables:
- * tab-width: 4
- * c-basic-offset: 4
- * End:
- * vim600: noet sw=4 ts=4 fdm=marker
- * vim<600: noet sw=4 ts=4
- */
