@@ -14,18 +14,24 @@
  * limitations under the License.
  */
 
+#include "bson/bson.h"
+#include "mongoc/mongoc.h"
+
 #include <php.h>
 #include <Zend/zend_interfaces.h>
 
-#ifdef HAVE_CONFIG_H
-#include "config.h"
-#endif
-
-#include "phongo_compat.h"
-#include "php_phongo.h"
-#include "php_bson.h"
 #include "php_array_api.h"
-#include "Session.h"
+
+#include "php_phongo.h"
+#include "phongo_bson_encode.h"
+#include "phongo_client.h"
+#include "phongo_error.h"
+
+#include "MongoDB/ReadConcern.h"
+#include "MongoDB/ReadPreference.h"
+#include "MongoDB/Server.h"
+#include "MongoDB/Session.h"
+#include "MongoDB/WriteConcern.h"
 
 zend_class_entry* php_phongo_session_ce;
 
@@ -587,6 +593,21 @@ static PHP_METHOD(Session, endSession)
 	intern->client_session = NULL;
 } /* }}} */
 
+/* {{{ proto bool MongoDB\Driver\Session::isDirty()
+   Returns whether the session is dirty (i.e. was used with a command that
+   encountered a network error) and will be discarded when returned to the
+   server session pool. */
+static PHP_METHOD(Session, isDirty)
+{
+	php_phongo_session_t* intern = Z_SESSION_OBJ_P(getThis());
+
+	SESSION_CHECK_LIVELINESS(intern, "isDirty")
+
+	PHONGO_PARSE_PARAMETERS_NONE();
+
+	RETVAL_BOOL(mongoc_client_session_get_dirty(intern->client_session));
+} /* }}} */
+
 /* {{{ proto void MongoDB\Driver\Session::isInTransaction(void)
    Returns whether a multi-document transaction is in progress */
 static PHP_METHOD(Session, isInTransaction)
@@ -636,6 +657,7 @@ static zend_function_entry php_phongo_session_me[] = {
 	PHP_ME(Session, getServer, ai_Session_void, ZEND_ACC_PUBLIC | ZEND_ACC_FINAL)
 	PHP_ME(Session, getTransactionOptions, ai_Session_void, ZEND_ACC_PUBLIC | ZEND_ACC_FINAL)
 	PHP_ME(Session, getTransactionState, ai_Session_void, ZEND_ACC_PUBLIC | ZEND_ACC_FINAL)
+	PHP_ME(Session, isDirty, ai_Session_void, ZEND_ACC_PUBLIC | ZEND_ACC_FINAL)
 	PHP_ME(Session, isInTransaction, ai_Session_void, ZEND_ACC_PUBLIC | ZEND_ACC_FINAL)
 	PHP_ME(Session, startTransaction, ai_Session_startTransaction, ZEND_ACC_PUBLIC | ZEND_ACC_FINAL)
 	ZEND_NAMED_ME(__construct, PHP_FN(MongoDB_disabled___construct), ai_Session_void, ZEND_ACC_PRIVATE | ZEND_ACC_FINAL)
@@ -672,9 +694,7 @@ static void php_phongo_session_free_object(zend_object* object) /* {{{ */
 
 static zend_object* php_phongo_session_create_object(zend_class_entry* class_type) /* {{{ */
 {
-	php_phongo_session_t* intern = NULL;
-
-	intern = PHONGO_ALLOC_OBJECT_T(php_phongo_session_t, class_type);
+	php_phongo_session_t* intern = zend_object_alloc(sizeof(php_phongo_session_t), class_type);
 
 	zend_object_std_init(&intern->std, class_type);
 	object_properties_init(&intern->std, class_type);
@@ -696,28 +716,28 @@ static HashTable* php_phongo_session_get_debug_info(phongo_compat_object_handler
 
 	array_init(&retval);
 
-	if (intern->client_session) {
-		const bson_t*         lsid;
+	if (!intern->client_session) {
+		ADD_ASSOC_BOOL_EX(&retval, "ended", true);
+
+		goto done;
+	}
+
+	{
+		const bson_t*         lsid = mongoc_client_session_get_lsid(intern->client_session);
 		php_phongo_bson_state state;
 
 		PHONGO_BSON_INIT_DEBUG_STATE(state);
 
-		lsid = mongoc_client_session_get_lsid(intern->client_session);
-
 		if (!php_phongo_bson_to_zval_ex(bson_get_data(lsid), lsid->len, &state)) {
 			zval_ptr_dtor(&state.zchild);
 			goto done;
 		}
 
 		ADD_ASSOC_ZVAL_EX(&retval, "logicalSessionId", &state.zchild);
-	} else {
-		ADD_ASSOC_NULL_EX(&retval, "logicalSessionId");
 	}
 
-	if (intern->client_session) {
-		const bson_t* cluster_time;
-
-		cluster_time = mongoc_client_session_get_cluster_time(intern->client_session);
+	{
+		const bson_t* cluster_time = mongoc_client_session_get_cluster_time(intern->client_session);
 
 		if (cluster_time) {
 			php_phongo_bson_state state;
@@ -733,18 +753,15 @@ static HashTable* php_phongo_session_get_debug_info(phongo_compat_object_handler
 		} else {
 			ADD_ASSOC_NULL_EX(&retval, "clusterTime");
 		}
-	} else {
-		ADD_ASSOC_NULL_EX(&retval, "clusterTime");
 	}
 
-	if (intern->client_session) {
+	{
 		const mongoc_session_opt_t* cs_opts = mongoc_client_session_get_opts(intern->client_session);
 		ADD_ASSOC_BOOL_EX(&retval, "causalConsistency", mongoc_session_opts_get_causal_consistency(cs_opts));
-	} else {
-		ADD_ASSOC_NULL_EX(&retval, "causalConsistency");
+		ADD_ASSOC_BOOL_EX(&retval, "snapshot", mongoc_session_opts_get_snapshot(cs_opts));
 	}
 
-	if (intern->client_session) {
+	{
 		uint32_t timestamp, increment;
 
 		mongoc_client_session_get_operation_time(intern->client_session, &timestamp, &increment);
@@ -757,15 +774,12 @@ static HashTable* php_phongo_session_get_debug_info(phongo_compat_object_handler
 		} else {
 			ADD_ASSOC_NULL_EX(&retval, "operationTime");
 		}
-	} else {
-		ADD_ASSOC_NULL_EX(&retval, "operationTime");
 	}
 
-	if (intern->client_session) {
+	{
 		uint32_t server_id = mongoc_client_session_get_server_id(intern->client_session);
 
 		if (server_id) {
-
 			zval server;
 
 			phongo_server_init(&server, &intern->manager, server_id);
@@ -773,17 +787,12 @@ static HashTable* php_phongo_session_get_debug_info(phongo_compat_object_handler
 		} else {
 			ADD_ASSOC_NULL_EX(&retval, "server");
 		}
-	} else {
-		ADD_ASSOC_NULL_EX(&retval, "server");
 	}
 
-	if (intern->client_session) {
-		ADD_ASSOC_BOOL_EX(&retval, "inTransaction", mongoc_client_session_in_transaction(intern->client_session));
-	} else {
-		ADD_ASSOC_NULL_EX(&retval, "inTransaction");
-	}
+	ADD_ASSOC_BOOL_EX(&retval, "dirty", mongoc_client_session_get_dirty(intern->client_session));
+	ADD_ASSOC_BOOL_EX(&retval, "inTransaction", mongoc_client_session_in_transaction(intern->client_session));
 
-	if (intern->client_session) {
+	{
 		const char* state = php_phongo_get_transaction_state_string(mongoc_client_session_get_transaction_state(intern->client_session));
 
 		if (!state) {
@@ -792,16 +801,13 @@ static HashTable* php_phongo_session_get_debug_info(phongo_compat_object_handler
 		}
 
 		ADD_ASSOC_STRING(&retval, "transactionState", state);
-	} else {
-		ADD_ASSOC_NULL_EX(&retval, "transactionState");
 	}
 
-	if (intern->client_session) {
+	{
 		zval txn_opts;
+
 		php_phongo_transaction_options_to_zval(intern->client_session, &txn_opts);
 		ADD_ASSOC_ZVAL_EX(&retval, "transactionOptions", &txn_opts);
-	} else {
-		ADD_ASSOC_NULL_EX(&retval, "transactionOptions");
 	}
 
 done:
@@ -831,11 +837,15 @@ void php_phongo_session_init_ce(INIT_FUNC_ARGS) /* {{{ */
 	zend_declare_class_constant_string(php_phongo_session_ce, ZEND_STRL("TRANSACTION_ABORTED"), PHONGO_TRANSACTION_ABORTED);
 } /* }}} */
 
-/*
- * Local variables:
- * tab-width: 4
- * c-basic-offset: 4
- * End:
- * vim600: noet sw=4 ts=4 fdm=marker
- * vim<600: noet sw=4 ts=4
- */
+void phongo_session_init(zval* return_value, zval* manager, mongoc_client_session_t* client_session) /* {{{ */
+{
+	php_phongo_session_t* session;
+
+	object_init_ex(return_value, php_phongo_session_ce);
+
+	session                 = Z_SESSION_OBJ_P(return_value);
+	session->client_session = client_session;
+
+	ZVAL_ZVAL(&session->manager, manager, 1, 0);
+}
+/* }}} */
