@@ -10,6 +10,34 @@ use MongoDB\Driver\Exception\RuntimeException;
 require_once __DIR__ . '/basic.inc';
 require_once __DIR__ . '/tools.php';
 
+/**
+ * Disables SKIPIF caching (PHP 8.1+).
+ */
+function disable_skipif_caching()
+{
+    if (PHP_VERSION_ID < 80100) {
+        return;
+    }
+
+    echo "nocache\n";
+}
+
+/**
+ * Skips the test if the topology is load balanced.
+ */
+function skip_if_load_balanced()
+{
+    is_load_balanced(URI) and exit('skip topology is load balanced');
+}
+
+/**
+ * Skips the test if the topology is not load balanced.
+ */
+function skip_if_not_load_balanced()
+{
+    is_load_balanced(URI) or exit('skip topology is not load balanced');
+}
+
 /**
  * Skips the test if the topology is a sharded cluster.
  */
@@ -48,9 +76,9 @@ function skip_if_not_mongos()
     is_mongos(URI) or exit('skip topology is not a sharded cluster');
 }
 
-function skip_if_not_mongos_with_replica_set()
+function skip_if_not_sharded_cluster_with_replica_set()
 {
-    is_mongos_with_replica_set(URI) or exit('skip topology is not a sharded cluster with replica set');
+    is_sharded_cluster_with_replica_set(URI) or exit('skip topology is not a sharded cluster with replica set');
 }
 
 /**
@@ -72,14 +100,14 @@ function skip_if_not_replica_set()
 /**
  * Skips the test if the topology is not a replica set or sharded cluster backed by replica sets
  */
-function skip_if_not_replica_set_or_mongos_with_replica_set()
+function skip_if_not_replica_set_or_sharded_cluster_with_replica_set()
 {
-    is_replica_set(URI) or is_mongos_with_replica_set(URI) or exit('skip topology is not a replica set or sharded cluster with replica set');
+    is_replica_set(URI) or is_sharded_cluster_with_replica_set(URI) or exit('skip topology is not a replica set or sharded cluster with replica set');
 }
 
 function skip_if_no_transactions()
 {
-    if (is_mongos_with_replica_set(URI)) {
+    if (is_sharded_cluster_with_replica_set(URI)) {
         skip_if_server_version('<', '4.2');
     } elseif (is_replica_set(URI)) {
         skip_if_server_version('<', '4.0');
@@ -404,16 +432,17 @@ function skip_if_not_clean($databaseName = DATABASE_NAME, $collectionName = COLL
     } catch (RuntimeException $e) {
         exit("skip Could not drop '$databaseName.$collectionName': " . $e->getMessage());
     }
+
+    /* Since this function modifies the state of the database, we need it to run
+     * each time before a test. */
+    disable_skipif_caching();
 }
 
 function skip_if_no_getmore_failpoint()
 {
     $serverVersion = get_server_version(URI);
 
-    if (
-        version_compare($serverVersion, '3.2', '>=') &&
-        version_compare($serverVersion, '4.0', '<')
-    ) {
+    if (version_compare($serverVersion, '4.0', '<')) {
         exit("skip Server version '$serverVersion' does not support a getMore failpoint'");
     }
 }
